"""
YahooView.py by ddt<ddt@mirstyle.org>
license: GPL
"""
import gtk
import os
import urllib
import HTMLParser
import re
import threading
import Queue
import YahooDict
import webbrowser

class YahooView(HTMLParser.HTMLParser, threading.Thread):
    replace_list = ((re.compile("rgb\(00,80,00\)", re.I),'#008000'),
                    (re.compile(r"\n"),""),
                    (re.compile("<br>"),"\n"),
                    (re.compile("</?table[^>]*>"), ""),
                    (re.compile("</?td[^>]*>"), ""),
                    (re.compile("</?tr[^>]*>"), ""),
                    (re.compile("</?span[^>]*>"), ""),
                    (re.compile("</?p[^>]*>"), "\n"),
                    (re.compile("(&#[0-9]+)([^0-9])"), "\g<1>;\g<2>")
                    )
    
    del_js_p = re.compile("\'([^']+)\'")
    
    def __init__(self, app, view):
        HTMLParser.HTMLParser.__init__(self)
        threading.Thread.__init__(self)
        self.queue = Queue.Queue(3)
        self.dict = YahooDict.YahooDict()
        self.app = app
        self.view = view
        self.buffer = self.view.get_buffer()
        self.font_tree = []
        self.link = ''
        self.on_b = False
        self.bold = self.buffer.create_tag()
        self.bold.set_property('weight',700)
        h = os.getenv('HOME')
        if h == None: # win32
            h = 'C:\\'
        self.home = h+'/.gdick/'
        if not os.path.exists(self.home):
            os.mkdir(self.home)

    def run(self):
        while 1:
            word, lang = self.queue.get()
            if lang == "end": #shutdown
                print "end"
                break
            r = self.dict.search(word, lang)
            self.show(r)
    
    def search(self, word, lang):
        self.queue.put((word,lang))

    def show(self, html_data):
        for p, s in self.replace_list:
            html_data = p.sub(s, html_data)
        html_data= unicode(html_data, 'euc-kr','ignore')
        gtk.threads_enter()
        start = self.buffer.get_start_iter()
        self.buffer.insert(start, '\n') # margin
        try:
            self.feed(html_data)
            self.close()
        except HTMLParser.HTMLParseError:
            pass
        gtk.threads_leave()

    def download(self,url):
        name = self.home + os.path.basename(url)
        if not os.path.exists(name):
            f = open(name,'w')
            data = urllib.urlopen(url).read()
            f.write(data)
            f.close()
        pixbuf = gtk.gdk.pixbuf_new_from_file(name)
        end = self.buffer.get_end_iter()
        self.buffer.insert_pixbuf(end, pixbuf)

    def link_press_cb(self, widget, event, word):
        if not word[-1].isdigit(): #if not sound link
            self.app.search(word)
        else:
            url = "http://msrv.yahoo.co.kr/engdic/sound/"
            url += word[0]
            url += "/" + word + ".wma"
            webbrowser.open_new(url)
    
    # HTMLParse methods
    def handle_starttag(self, name, attrs):
        if name == 'font':
            tag = self.buffer.create_tag()
            for attr, value in attrs:
                if attr == 'color':
                    tag.set_property('foreground',value)
            self.font_tree.append(tag)
        elif name == 'b':
            self.on_b = True
        else:
            for attr, value in attrs:
                if attr == 'href':
                    m = self.del_js_p.findall(value)
                    if m:
                        self.link = m[-1]
                    break
                elif attr == 'src':
                    self.download(value)
                    break

    def handle_endtag(self, tag):
        if tag == 'a':
            self.link = ''
        elif tag == 'font':
            try:
                del self.font_tree[-1]
            except IndexError: # malformed html :(
                pass
        elif tag == 'b':
            self.on_b = False

    def handle_data(self, data):
        end = self.buffer.get_end_iter()
        if self.link:
            l = gtk.Label('<span background="white">'+data+'</span>')
            l.set_use_markup(gtk.TRUE)
            e = gtk.EventBox()
            e.connect("button-press-event",self.link_press_cb, self.link)
            e.add(l)
            e.show_all()
            anchor = self.buffer.create_child_anchor(end)
            self.view.add_child_at_anchor(e,anchor)
            return
        if self.font_tree != []:
            tag = self.font_tree[-1]
            self.buffer.insert_with_tags(end, data, tag)
            return
        if self.on_b:
            self.buffer.insert_with_tags(end, data, self.bold)
            return
        self.buffer.insert(end, data)

    def handle_charref(self, name):
        print name
        end = self.buffer.get_end_iter()
        self.buffer.insert(end, unichr(int(name)))

    def handle_entityref(self, name):
        print name
        end = self.buffer.get_end_iter()
        if name == 'nbsp':
            self.buffer.insert(end, ' ')
